#! /usr/bin/env python3

global PROJECT_NAME
PROJECT_NAME="PolygonCrest"



import sys
import os
import subprocess
import shutil



def YsAtoi(s):
	state=0
	value=0
	#state: 0 Searching a number    1:Looking at a numeric character
	for i in range(len(s)):
		if 0==state:
			if '0'<=s[i] and s[i]<='9':
				state=1
				value=ord(s[i])-ord("0")
		else:
			if '0'<=s[i] and s[i]<='9':
				value*=10
				value+=ord(s[i])-ord("0")
			else:
				break

	return value



def YsMkDir(path):
	path=os.path.expanduser(path)
	if not os.path.isdir(path):
		try:
			os.makedirs(path)
		except:
			print("Fatal Error: Cannot create a temporary directory.")
			quit()



def YsForceDelete(path):
	try:
		print('Deleting '+path)
		if os.path.isdir(path):
			shutil.rmtree(path)
		elif os.path.exists(path):
			os.remove(path)
	except:
		print("Fatal Error: Cannot delete a file or a directory.")
		quit()



#outOption can be:  "/Fe" for Visual C++, or "-o " for gcc
def YsGetBitnessFromCompiler(workDir,compiler,outOption):
	YsMkDir(workDir)

	prog='#include <stdio.h>\nint main(void){printf("%d\\n",8*(int)sizeof(void *));return 8*(int)sizeof(void *);}'
	sourceFile=workDir+'/'+'ysbitnesstest.c'
	exeFile=workDir+'/'+'ysbitnesstest.exe'
	sourceFile=os.path.expanduser(sourceFile)
	exeFile=os.path.expanduser(exeFile)

	try:
		print("Writing source")
		f=open(sourceFile,'w')
		f.write(prog)
		f.close()
		compileCmd=compiler+" "+sourceFile+" "+outOption+exeFile
		print(compileCmd)
		os.system(compileCmd)
	except:
		print("Fatal Error: Cannot test bitness.")
		print("Either C compiler not available or cannot generate a temporary source file.")
		quit()

	# Note: The return value of os.system is system-dependent 
	#       according to the official python web site
	#       Furthermore, os.system is just using C system function.
	print("Testing compiler bitness")
	process=subprocess.Popen([exeFile],stdout=subprocess.PIPE)
	bitnessOut=process.communicate()[0]
	bitnessStr=bitnessOut.decode("utf-8")
	bitness=YsAtoi(str(bitnessStr))
	print("Returned Bitness=",bitness)
	return bitness



def YsSvnRevision():
	process=subprocess.Popen(["svnversion"],stdout=subprocess.PIPE)
	rawOutput=process.communicate()[0]

	#consoleOutput is a byte literal, and needs to be decoded into a string.
	#http://stackoverflow.com/questions/606191/convert-byte-array-to-python-string
	outputStr=str(rawOutput.decode("utf-8"))
	wordArray=outputStr.split("'")
	intValue=YsAtoi(str(wordArray[0]))

	return intValue;



def YsSvnExport(SRC_ROOT,DST_ROOT,moduleName):
	cmdStr="svn export "+SRC_ROOT+"/"+moduleName+" "+DST_ROOT+"/src/"+moduleName
	print(cmdStr)
	os.system(cmdStr)




#################
# Program entry #
#################

#main

if sys.platform.startswith('linux'):
	print("Linux platform")
	OS="Linux"
	DIST_ROOT=os.path.expanduser("~/Distribution/Linux")
	TMPDIR=os.path.expanduser("~/tmp")
	CC="g++"
	CCOUTOPTION="-o "
	ZIP='zip -r '
	makecmd='make -f makefile.unx'
elif sys.platform.startswith('darwin'):
	print("Mac OSX")
	OS="MacOSX"
	DIST_ROOT=os.path.expanduser("~/Distribution/MacOSX")
	TMPDIR=os.path.expanduser("~/tmp")
	CC="clang"
	CCOUTOPTION="-o "
	ZIP='zip -r '
	makecmd='make -f makefile.unx'
elif sys.platform.startswith('win') or sys.platform.startswith('cygwin'):
	print("Windows")
	OS="Win"
	DIST_ROOT="C:/tmp/Distribution/Win"
	TMPDIR="c:/tmp"
	CC="cl"
	CCOUTOPTION="/Fe"
	ZIP='7z a '
	makecmd='make -f makefile.win'
else:
	print("What is this system?  ",sys.platform)
	quit()

#bitness=YsGetBitnessFromCompiler(TMPDIR,CC,CCOUTOPTION)

#DIST_ROOT+=str(bitness)

print("OS=",OS)
#print("Bitness=",bitness)
print("DIST_ROOT=",DIST_ROOT)



svnversion=YsSvnRevision()
print("svnversion returned ",svnversion)



cwd=os.getcwd()
os.chdir('../svnrevision')
os.system(makecmd)
os.chdir(cwd)


SRC_ROOT='..'
DST_ROOT=DIST_ROOT+'/'+PROJECT_NAME

DST_SRC=DST_ROOT+'/src'
DST_BIN=DST_ROOT+'/bin'
YsForceDelete(DST_SRC)
YsForceDelete(DST_BIN)
YsMkDir(DST_SRC)



if OS=='Win':
	os.chdir('../svnrevision')
	os.system('nmake -f makefile.win')
	os.chdir(cwd)
	os.environ['LINKSUBSYS']='WINDOWS'
	os.system('rebuild_64_and_32.bat')
	shutil.copytree('runtime',DST_BIN)
	ZIP_ROOT=DIST_ROOT
elif OS=='Linux':
	objpath=os.path.expanduser('~/ysbin')
	os.system('rm -r '+objpath)
	os.chdir('../svnrevision')
	os.system('make -f makefile.unx')
	os.chdir(cwd)
	os.system('./rebuildLinux32and64.sh')
	shutil.copytree('runtime',DST_BIN)
	ZIP_ROOT='/ysshare/Distribution'
elif OS=='MacOSX':
	objpath=os.path.expanduser('~/ysbin')
	os.system('rm -r '+objpath)
	os.chdir('../svnrevision')
	os.system('make -f makefile.unx')
	os.chdir(cwd)
	os.chdir('src/gui')
	os.system('make -f makefile.unx')
	os.chdir(cwd)
	YsForceDelete(DST_ROOT+'/PolygonCrest.app')
	shutil.copytree('src/gui/PolygonCrest.app',DST_ROOT+'/PolygonCrest.app')
	ZIP_ROOT=DIST_ROOT
else:
	print("Fatal Error: OS is suppsoed to be Windows, Linux, or MacOSX")
	quit()



ARCHIVE_NAME=ZIP_ROOT+'/'+PROJECT_NAME+'_'+OS+'.zip'



print('Re-building done.')
#memo: input function is another thing that changed from Python2.x to 3.x
#      Very annoying.  Another indication of fragile language foundation of Python.
#      Someone who defines the language specification must not casully change it.
#      The following try and catch is necessary to make it compatible for Python 2 and 3.
try:
        input('Press Enter to Continue:')
except:
        print('.')


shutil.copy('readme_English.txt',DST_ROOT)
shutil.copy('readme_Japanese.txt',DST_ROOT)



YsSvnExport(SRC_ROOT,DST_ROOT,'ysgebl')
YsForceDelete(DST_SRC+'/ysgebl/memo')
YsForceDelete(DST_SRC+'/ysgebl/src/debug')
YsForceDelete(DST_SRC+'/ysgebl/src/test')
YsForceDelete(DST_SRC+'/ysgebl/src/gui-template')
YsForceDelete(DST_SRC+'/ysgebl/ideas.txt')
YsForceDelete(DST_SRC+'/ysgebl/makedistribution.bat')
YsForceDelete(DST_SRC+'/ysgebl/runtime/ysgebl.exe')
YsForceDelete(DST_SRC+'/ysgebl/runtime/polygoncrest')
YsForceDelete(DST_SRC+'/ysgebl/src/polygoncrest')
YsForceDelete(DST_SRC+'/ysgebl/src/polygoncrest.app')

YsSvnExport(SRC_ROOT,DST_ROOT,'ysclass')
YsForceDelete(DST_SRC+'/ysclass/*.bat')
YsForceDelete(DST_SRC+'/ysclass/memo')
YsForceDelete(DST_SRC+'/ysclass/test')
YsForceDelete(DST_SRC+'/ysclass/test_comprehensive')
YsForceDelete(DST_SRC+'/ysclass/test_graphical')
YsForceDelete(DST_SRC+'/ysclass/util')

YsSvnExport(SRC_ROOT,DST_ROOT,'imported')

YsSvnExport(SRC_ROOT,DST_ROOT,'fsguilib')
YsForceDelete(DST_SRC+'/fsguilib/memo')
YsForceDelete(DST_SRC+'/fsguilib/2dsample')
YsForceDelete(DST_SRC+'/fsguilib/3dsample')
YsForceDelete(DST_SRC+'/fsguilib/apptemplate')
YsForceDelete(DST_SRC+'/fsguilib/test')

YsSvnExport(SRC_ROOT,DST_ROOT,'fsgui3d')
YsForceDelete(DST_SRC+'/fsgui3d/png')
YsForceDelete(DST_SRC+'/fsgui3d/test')
YsForceDelete(DST_SRC+'/fsgui3d/export.bat')

YsSvnExport(SRC_ROOT,DST_ROOT,'fssimplewindow')
YsForceDelete(DST_SRC+'/fssimplewindow/test')
YsForceDelete(DST_SRC+'/fssimplewindow/errorTest')
YsForceDelete(DST_SRC+'/fssimplewindow/memo')

YsSvnExport(SRC_ROOT,DST_ROOT,'ysbitmap')
YsForceDelete(DST_SRC+'/ysbitmap/filter')
YsForceDelete(DST_SRC+'/ysbitmap/memo')
YsForceDelete(DST_SRC+'/ysbitmap/test')
YsForceDelete(DST_SRC+'/ysbitmap/pngencoderdist')

YsSvnExport(SRC_ROOT,DST_ROOT,'ysgl')
YsForceDelete(DST_SRC+'/ysgl/experiment')
YsForceDelete(DST_SRC+'/ysgl/memo')
YsForceDelete(DST_SRC+'/ysgl/test')
YsForceDelete(DST_SRC+'/ysgl/test_iOS')

YsSvnExport(SRC_ROOT,DST_ROOT,'ysglcpp')

YsSvnExport(SRC_ROOT,DST_ROOT,'ysbitmapfont')
YsForceDelete(DST_SRC+'/ysbitmapfont/bwconv')
YsForceDelete(DST_SRC+'/ysbitmapfont/memo')
YsForceDelete(DST_SRC+'/ysbitmapfont/test')
YsForceDelete(DST_SRC+'/ysbitmapfont/test1')
YsForceDelete(DST_SRC+'/ysbitmapfont/test2')
YsForceDelete(DST_SRC+'/ysbitmapfont/test3')
YsForceDelete(DST_SRC+'/ysbitmapfont/test4')

YsSvnExport(SRC_ROOT,DST_ROOT,'ysfontrenderer')

YsSvnExport(SRC_ROOT,DST_ROOT,'yssystemfont')
YsForceDelete(DST_SRC+'/yssystemfont/memo')
YsForceDelete(DST_SRC+'/yssystemfont/test')

YsSvnExport(SRC_ROOT,DST_ROOT,'ysport')
YsForceDelete(DST_SRC+'/ysport/experiment')
YsForceDelete(DST_SRC+'/ysport/memo')
YsForceDelete(DST_SRC+'/ysport/test')


YsMkDir(DST_SRC+'/svnrevision')
shutil.copy2(SRC_ROOT+'/svnrevision/yssvnrevision.h',DST_SRC+'/svnrevision/.')


YsMkDir(ZIP_ROOT)

print(ARCHIVE_NAME)
os.chdir(DIST_ROOT)   #(Distribution)/

YsForceDelete(ARCHIVE_NAME)
zipCmd=ZIP+ARCHIVE_NAME+' '+PROJECT_NAME
print(zipCmd)
os.system(zipCmd)
print("Done: "+zipCmd)
