/* ////////////////////////////////////////////////////////////

  File Name: ysshellextmisc.h
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#ifndef YSSHELLEXTMISC_IS_INCLUDED
#define YSSHELLEXTMISC_IS_INCLUDED
/* { */

#include "ysshellext.h"

class YsShellExt_VertexInsertionInfo
{
public:
	YsShellVertexHandle beforeOrAfterThisVtHd;
	YsShellVertexHandle insertThisVtHd;
	int beforeOrAfter;        // Positive=>After  Negative=>Before
};

class YsShellExt_ConstEdgeVertexInsertionInfo : public YsShellExt_VertexInsertionInfo
{
public:
	YsShellExt::ConstEdgeHandle ceHd;
};

////////////////////////////////////////////////////////////

class YsShellExt::ConstEdgeStore : public YsKeyStore
{
friend class const_iterator;

public:
	/*! In case it needs to iterate through polygon keys, you can write like:
		YsShellPolygonStore plStore;
			for(auto iter=plStore.AllKey().begin(); iter!=plStore.AllKey().end(); ++iter)
			{
				// Do something useful
			}
		or,
			for(auto key : plStore.AllKey()
			{
				// Do something useful
			}
		using range-based for.
	*/
	const YsKeyStore &AllKey(void) const;

	/*! STL-like iterator support. */
	class const_iterator
	{
	friend class ConstEdgeStore;

	private:
		const ConstEdgeStore *table;
		YsKeyStore::const_iterator iter;
	public:
		inline const_iterator &operator++();
		inline const_iterator operator++(int);
		inline bool operator==(const const_iterator &from);
		inline bool operator!=(const const_iterator &from);
		inline const YsShellExt::ConstEdgeHandle operator*();  // Due to dependency, it is written in ysshellsearch.h
	};

	/*! Support for STL-like iterator */
	inline const_iterator begin() const;

	/*! Support for STL-like iterator */
	inline const_iterator end() const;



	ConstEdgeStore();
	ConstEdgeStore(const YsShellExt &shl);
	ConstEdgeStore(const YsShellExt &shl,YSSIZE_T nCe,const YsShellExt::ConstEdgeHandle ceHdArray[]);

	template <class T>
	ConstEdgeStore(const YsShellExt &shl,const T &hdStore);

	void CleanUp(void);
	void SetShell(const YsShellExt &shl);
	const YsShellExt &GetShell(void) const;
	YSRESULT AddConstEdge(YsShellExt::ConstEdgeHandle ceHd);
	YSRESULT AddConstEdge(YSSIZE_T nVt,const YsShellExt::ConstEdgeHandle ceHd[]);

	template <class T>
	YSRESULT AddConstEdge(const T &hdStore);

	YSRESULT DeleteConstEdge(YsShellExt::ConstEdgeHandle ceHd);
	YSBOOL IsIncluded(YsShellExt::ConstEdgeHandle ceHd) const;
protected:
	const YsShellExt *shl;
};

inline YsShellExt::ConstEdgeStore::const_iterator YsShellExt::ConstEdgeStore::begin() const
{
	const_iterator iter;
	iter.table=this;
	iter.iter=YsKeyStore::begin();
	return iter;
}

inline YsShellExt::ConstEdgeStore::const_iterator YsShellExt::ConstEdgeStore::end() const
{
	const_iterator iter;
	iter.table=this;
	iter.iter=YsKeyStore::end();
	return iter;
}

inline YsShellExt::ConstEdgeStore::const_iterator &YsShellExt::ConstEdgeStore::const_iterator::operator++()
{
	++iter;
	return *this;
}

inline YsShellExt::ConstEdgeStore::const_iterator YsShellExt::ConstEdgeStore::const_iterator::operator++(int)
{
	const_iterator copy=*this;
	++iter;
	return copy;
}


inline bool YsShellExt::ConstEdgeStore::const_iterator::operator==(const YsShellExt::ConstEdgeStore::const_iterator &from)
{
	return this->iter==from.iter;
}


inline bool YsShellExt::ConstEdgeStore::const_iterator::operator!=(const YsShellExt::ConstEdgeStore::const_iterator &from)
{
	return this->iter!=from.iter;
}

inline const YsShellExt::ConstEdgeHandle YsShellExt::ConstEdgeStore::const_iterator::operator*()
{
	return table->shl->FindConstEdge(*iter);
}

inline YsShellExt::ConstEdgeStore::const_iterator begin(const YsShellExt::ConstEdgeStore &store)
{
	return store.begin();
}

inline YsShellExt::ConstEdgeStore::const_iterator end(const YsShellExt::ConstEdgeStore &store)
{
	return store.end();
}

template <class T>
YsShellExt::ConstEdgeStore::ConstEdgeStore(const YsShellExt &shl,const T &hdStore)
{
	this->shl=&shl;
	EnableAutoResizing();
	for(auto ceHd : hdStore)
	{
		AddConstEdge(ceHd);
	}
}

template <class T>
YSRESULT YsShellExt::ConstEdgeStore::AddConstEdge(const T &hdStore)
{
	for(auto ceHd : hdStore)
	{
		AddConstEdge(ceHd);
	}
	return YSOK;
}

////////////////////////////////////////////////////////////

class YsShellExt::FaceGroupStore : public YsKeyStore
{
friend class const_iterator;

public:
	/*! In case it needs to iterate through polygon keys, you can write like:
		YsShellPolygonStore plStore;
			for(auto iter=plStore.AllKey().begin(); iter!=plStore.AllKey().end(); ++iter)
			{
				// Do something useful
			}
		or,
			for(auto key : plStore.AllKey()
			{
				// Do something useful
			}
		using range-based for.
	*/
	const YsKeyStore &AllKey(void) const;

	/*! STL-like iterator support. */
	class const_iterator
	{
	friend class FaceGroupStore;

	private:
		const FaceGroupStore *table;
		YsKeyStore::const_iterator iter;
	public:
		inline const_iterator &operator++();
		inline const_iterator operator++(int);
		inline bool operator==(const const_iterator &from);
		inline bool operator!=(const const_iterator &from);
		inline const YsShellExt::FaceGroupHandle operator*();  // Due to dependency, it is written in ysshellsearch.h
	};

	/*! Support for STL-like iterator */
	inline const_iterator begin() const;

	/*! Support for STL-like iterator */
	inline const_iterator end() const;



	FaceGroupStore();
	FaceGroupStore(const YsShellExt &shl);
	FaceGroupStore(const YsShellExt &shl,YSSIZE_T nCe,const YsShellExt::FaceGroupHandle fgHdArray[]);

	template <class T>
	FaceGroupStore(const YsShellExt &shl,const T &hdStore);

	void CleanUp(void);
	void SetShell(const YsShellExt &shl);
	const YsShellExt &GetShell(void) const;
	YSRESULT AddFaceGroup(YsShellExt::FaceGroupHandle fgHd);
	YSRESULT AddFaceGroup(YSSIZE_T nVt,const YsShellExt::FaceGroupHandle fgHd[]);

	template <class T>
	YSRESULT AddFaceGroup(const T &hdStore);

	YSRESULT DeleteFaceGroup(YsShellExt::FaceGroupHandle fgHd);
	YSBOOL IsIncluded(YsShellExt::FaceGroupHandle fgHd) const;
protected:
	const YsShellExt *shl;
};

inline YsShellExt::FaceGroupStore::const_iterator YsShellExt::FaceGroupStore::begin() const
{
	const_iterator iter;
	iter.table=this;
	iter.iter=YsKeyStore::begin();
	return iter;
}

inline YsShellExt::FaceGroupStore::const_iterator YsShellExt::FaceGroupStore::end() const
{
	const_iterator iter;
	iter.table=this;
	iter.iter=YsKeyStore::end();
	return iter;
}

inline YsShellExt::FaceGroupStore::const_iterator &YsShellExt::FaceGroupStore::const_iterator::operator++()
{
	++iter;
	return *this;
}

inline YsShellExt::FaceGroupStore::const_iterator YsShellExt::FaceGroupStore::const_iterator::operator++(int)
{
	const_iterator copy=*this;
	++iter;
	return copy;
}


inline bool YsShellExt::FaceGroupStore::const_iterator::operator==(const YsShellExt::FaceGroupStore::const_iterator &from)
{
	return this->iter==from.iter;
}


inline bool YsShellExt::FaceGroupStore::const_iterator::operator!=(const YsShellExt::FaceGroupStore::const_iterator &from)
{
	return this->iter!=from.iter;
}

inline const YsShellExt::FaceGroupHandle YsShellExt::FaceGroupStore::const_iterator::operator*()
{
	return table->shl->FindFaceGroup(*iter);
}

inline YsShellExt::FaceGroupStore::const_iterator begin(const YsShellExt::FaceGroupStore &store)
{
	return store.begin();
}

inline YsShellExt::FaceGroupStore::const_iterator end(const YsShellExt::FaceGroupStore &store)
{
	return store.end();
}

template <class T>
YsShellExt::FaceGroupStore::FaceGroupStore(const YsShellExt &shl,const T &hdStore)
{
	this->shl=&shl;
	EnableAutoResizing();
	for(auto fgHd : hdStore)
	{
		AddFaceGroup(fgHd);
	}
}

template <class T>
YSRESULT YsShellExt::FaceGroupStore::AddFaceGroup(const T &hdStore)
{
	for(auto fgHd : hdStore)
	{
		AddFaceGroup(fgHd);
	}
	return YSOK;
}

////////////////////////////////////////////////////////////

class YsShellExt::VolumeStore : public YsKeyStore
{
friend class const_iterator;

public:
	/*! In case it needs to iterate through polygon keys, you can write like:
		YsShellPolygonStore plStore;
			for(auto iter=plStore.AllKey().begin(); iter!=plStore.AllKey().end(); ++iter)
			{
				// Do something useful
			}
		or,
			for(auto key : plStore.AllKey()
			{
				// Do something useful
			}
		using range-based for.
	*/
	const YsKeyStore &AllKey(void) const;

	/*! STL-like iterator support. */
	class const_iterator
	{
	friend class VolumeStore;

	private:
		const VolumeStore *table;
		YsKeyStore::const_iterator iter;
	public:
		inline const_iterator &operator++();
		inline const_iterator operator++(int);
		inline bool operator==(const const_iterator &from);
		inline bool operator!=(const const_iterator &from);
		inline const YsShellExt::VolumeHandle operator*();  // Due to dependency, it is written in ysshellsearch.h
	};

	/*! Support for STL-like iterator */
	inline const_iterator begin() const;

	/*! Support for STL-like iterator */
	inline const_iterator end() const;



	VolumeStore();
	VolumeStore(const YsShellExt &shl);
	VolumeStore(const YsShellExt &shl,YSSIZE_T nCe,const YsShellExt::VolumeHandle vlHdArray[]);

	template <class T>
	VolumeStore(const YsShellExt &shl,const T &hdStore);

	void CleanUp(void);
	void SetShell(const YsShellExt &shl);
	const YsShellExt &GetShell(void) const;
	YSRESULT AddVolume(YsShellExt::VolumeHandle vlHd);
	YSRESULT AddVolume(YSSIZE_T nVt,const YsShellExt::VolumeHandle vlHd[]);

	template <class T>
	YSRESULT AddVolume(const T &hdStore);

	YSRESULT DeleteVolume(YsShellExt::VolumeHandle vlHd);
	YSBOOL IsIncluded(YsShellExt::VolumeHandle vlHd) const;
protected:
	const YsShellExt *shl;
};

inline YsShellExt::VolumeStore::const_iterator YsShellExt::VolumeStore::begin() const
{
	const_iterator iter;
	iter.table=this;
	iter.iter=YsKeyStore::begin();
	return iter;
}

inline YsShellExt::VolumeStore::const_iterator YsShellExt::VolumeStore::end() const
{
	const_iterator iter;
	iter.table=this;
	iter.iter=YsKeyStore::end();
	return iter;
}

inline YsShellExt::VolumeStore::const_iterator &YsShellExt::VolumeStore::const_iterator::operator++()
{
	++iter;
	return *this;
}

inline YsShellExt::VolumeStore::const_iterator YsShellExt::VolumeStore::const_iterator::operator++(int)
{
	const_iterator copy=*this;
	++iter;
	return copy;
}


inline bool YsShellExt::VolumeStore::const_iterator::operator==(const YsShellExt::VolumeStore::const_iterator &from)
{
	return this->iter==from.iter;
}


inline bool YsShellExt::VolumeStore::const_iterator::operator!=(const YsShellExt::VolumeStore::const_iterator &from)
{
	return this->iter!=from.iter;
}

inline const YsShellExt::VolumeHandle YsShellExt::VolumeStore::const_iterator::operator*()
{
	return table->shl->FindVolume(*iter);
}

inline YsShellExt::VolumeStore::const_iterator begin(const YsShellExt::VolumeStore &store)
{
	return store.begin();
}

inline YsShellExt::VolumeStore::const_iterator end(const YsShellExt::VolumeStore &store)
{
	return store.end();
}

template <class T>
YsShellExt::VolumeStore::VolumeStore(const YsShellExt &shl,const T &hdStore)
{
	this->shl=&shl;
	EnableAutoResizing();
	for(auto vlHd : hdStore)
	{
		AddVolume(vlHd);
	}
}

template <class T>
YSRESULT YsShellExt::VolumeStore::AddVolume(const T &hdStore)
{
	for(auto vlHd : hdStore)
	{
		AddVolume(vlHd);
	}
	return YSOK;
}

////////////////////////////////////////////////////////////

/* } */
#endif
